from typing import Any
try:
    from pymongo import MongoClient
except ImportError:
    MongoClient = None

from .abstract import HealthCheckAbstract
from ..models import HealthCheckStatusEnum


class HealthCheckMongoDb(HealthCheckAbstract):
    def __init__(self, host: str | None, port: str | None, **kwargs: Any):
        self.__host = host
        self.__port = port
        self.__kwargs = kwargs

    @property
    def service(self) -> str:
        return "mongodb"

    @property
    def connection_uri(self) -> str | None:
        return f"mongodb://{self.__host}:{self.__port}" if self.__host and self.__port else None

    @property
    def tags(self) -> list[str]:
        return ["mongodb", "database"]

    @property
    def comments(self) -> list[str]:
        return []

    def check_health(self) -> HealthCheckStatusEnum:
        if MongoClient is None:
            raise ImportError("pymongo is not installed. Install it with `pip install fastapi-healthz[mongodb]`.")

        res: HealthCheckStatusEnum = HealthCheckStatusEnum.UNHEALTHY

        try:
            client = MongoClient(host=self.__host, port=self.__port, **self.__kwargs)
            client.server_info()
            res = HealthCheckStatusEnum.HEALTHY
        except Exception:
            pass

        return res



from ssl import SSLContext, PROTOCOL_TLSv1_2
try:
    import pika
except ImportError:
    pika = None

from .abstract import HealthCheckAbstract
from ..models import HealthCheckStatusEnum


class HealthCheckRabbitMQ(HealthCheckAbstract):
    def __init__(
        self,
        host: str,
        vhost: str | None = "/",
        username: str | None = None,
        password: str | None = None,
        ssl: bool = True,
        port: int | None = 5672,
    ):
        self.__host = host
        self.__port = port
        self.__vhost = vhost
        self.__username = username
        self.__password = password
        self.__ssl = ssl

    @property
    def service(self) -> str:
        return "rabbitmq"

    @property
    def connection_uri(self) -> str | None:
        secure = "s" if self.__ssl else ""
        return f"amqp{secure}://{self.__host}:{str(self.__port)}"

    @property
    def tags(self) -> list[str]:
        return ["rabbitmq"]

    @property
    def comments(self) -> list[str]:
        return []

    def check_health(self) -> HealthCheckStatusEnum:
        if pika is None:
            raise ImportError("pika is not installed. Install it with `pip install fastapi-healthz[rabbitmq]`.")

        res: HealthCheckStatusEnum = HealthCheckStatusEnum.UNHEALTHY
        try:
            parameters = pika.ConnectionParameters(
                host=self.__host,
                port=self.__port,
                virtual_host=self.__vhost,
                credentials=pika.PlainCredentials(username=self.__username, password=self.__password),
            )

            if self.__ssl:
                # SSL Context for TLS configuration of Amazon MQ for RabbitMQ
                ssl_context = SSLContext(PROTOCOL_TLSv1_2)
                ssl_context.set_ciphers("ECDHE+AESGCM:!ECDSA")

                parameters.ssl_options = pika.SSLOptions(context=ssl_context)

            channel = pika.BlockingConnection(parameters).channel()
            if channel.connection.is_open:
                res = HealthCheckStatusEnum.HEALTHY
        except Exception:
            pass

        return res



from abc import ABC, abstractmethod
from ..models import HealthCheckStatusEnum


class HealthCheckAbstract(ABC):
    @abstractmethod
    def check_health(self) -> HealthCheckStatusEnum:
        """Requests data from the endpoint to validate health."""
        pass

    @property
    @abstractmethod
    def service(self) -> str:
        pass

    @property
    @abstractmethod
    def connection_uri(self) -> str | None:
        pass

    @property
    @abstractmethod
    def tags(self) -> list[str]:
        pass

    @property
    @abstractmethod
    def comments(self) -> list[str]:
        pass



try:
    from sqlalchemy import create_engine, text
    from sqlalchemy.orm import sessionmaker
except ImportError:
    create_engine = None
    text = None
    sessionmaker = None

from .abstract import HealthCheckAbstract
from ..models import HealthCheckStatusEnum


class HealthCheckDatabase(HealthCheckAbstract):
    def __init__(self, uri: str):
        self.__uri = uri

    @property
    def service(self) -> str:
        return "db"

    @property
    def connection_uri(self) -> str | None:
        return self.__uri

    @property
    def tags(self) -> list[str]:
        return ["database"]

    @property
    def comments(self) -> list[str]:
        return []

    def check_health(self) -> HealthCheckStatusEnum:
        if create_engine is None or text is None or sessionmaker is None:
            raise ImportError("SQLAlchemy is not installed. Install it with `pip install fastapi-healthz[database]`.")

        res: HealthCheckStatusEnum = HealthCheckStatusEnum.UNHEALTHY

        engine = create_engine(self.__uri)
        Session = sessionmaker(autocommit=False, autoflush=False, bind=engine)
        with Session() as session:
            try:
                session.execute(text("SELECT 1"))
                res = HealthCheckStatusEnum.HEALTHY
            except Exception:
                pass
        return res



try:
    from requests import get
except ImportError:
    get = None

from ..models import HealthCheckStatusEnum
from .abstract import HealthCheckAbstract


class HealthCheckUri(HealthCheckAbstract):
    def __init__(
        self,
        uri: str,
        service: str | None = None,
        tags: list[str] | None = None,
        healthy_code: int = 200,
        unhealthy_code: int = 500
    ) -> None:
        super().__init__(service=service, tags=tags)

        self.__connection_uri = uri
        self.__healthy_code = healthy_code
        self.__unhealthy_code = unhealthy_code

    @property
    def service(self) -> str:
        return self._service if self._service is not None else "uri"

    @property
    def connection_uri(self) -> str | None:
        return self.__connection_uri

    def check_health(self) -> HealthCheckStatusEnum:
        if get is None:
            raise ImportError("requests is not installed. Install it with `pip install fastapi-healthz[uri]`.")

        res = get(url=self.__connection_uri, headers={"User-Agent": "FastAPI HealthCheck"})
        if res.status_code == self.__healthy_code:
            return HealthCheckStatusEnum.HEALTHY
        elif res.status_code != self.__unhealthy_code:
            return HealthCheckStatusEnum.UNHEALTHY
        return HealthCheckStatusEnum.UNHEALTHY
